package handlers

import (
	"encoding/json"
	"strconv"

	"github.com/gofiber/fiber/v2"

	"gitlab.larvit.se/power-plan/auth-ui/src/api"
	"gitlab.larvit.se/power-plan/auth-ui/src/views"
)

func (h Handlers) Register(c *fiber.Ctx) error {
	c.Set(fiber.HeaderContentType, fiber.MIMETextHTML)
	return c.SendString(views.Register(views.RegisterData{}))
}

type field struct {
	Name   string   `json:"name"`
	Values []string `json:"values"`
}
type newUserType struct {
	Name           string  `json:"name"`
	Username       string  `json:"username"`
	Password       string  `json:"password"`
	RepeatPassword string  `json:"-"`
	Fields         []field `json:"fields"`
}

func (h Handlers) RegisterPost(c *fiber.Ctx) error {
	c.Set(fiber.HeaderContentType, fiber.MIMETextHTML)
	newUser := new(newUserType)

	err := c.BodyParser(newUser)
	if err != nil {
		h.Log.Debug("Invalid input data", "err", err.Error())
		c.Status(400)
	}
	newUser.Name = newUser.Username
	newUser.Fields = append(newUser.Fields, field{
		Name:   "role",
		Values: []string{"user"},
	})

	jsonPayload, err := json.Marshal(newUser)
	if err != nil {
		h.Log.Error("Could not marshal newUser struct", "err", err.Error())
		return c.Status(500).SendString(views.Error500())
	}

	apiRes, err := h.Api.Call("POST", "/account", jsonPayload)
	if err != nil {
		h.Log.Error("Could not call backend API", "err", err.Error())
		return c.Status(500).SendString(views.Error500())
	}

	h.Log.Debug("API res status", "status", apiRes.StatusCode)

	if apiRes.StatusCode == 201 {
		return c.SendString(views.Register(views.RegisterData{
			OkMsg: "New user created!",
		}))
	} else if strconv.Itoa(apiRes.StatusCode)[0:1] == "4" {
		var jsonResp []api.ApiResError

		err := json.Unmarshal(apiRes.Body, &jsonResp)
		if err != nil {
			h.Log.Error("Could not unmarshal api error response", "err", err.Error())
			return c.Status(500).SendString(views.Error500())
		}

		return c.Status(apiRes.StatusCode).SendString(views.Register(views.RegisterData{
			ErrField: jsonResp[0].Field,
			ErrMsg:   jsonResp[0].Error,
		}))
	} else {
		h.Log.Error("Unexpected API response status", "status", apiRes.StatusCode)
		return c.Status(500).SendString(views.Error500())
	}
}
