package main

import (
	"os"
	"time"

	"github.com/gofiber/fiber/v2"
	"github.com/joho/godotenv"
	"gitlab.larvit.se/power-plan/auth-ui/src/api"
	h "gitlab.larvit.se/power-plan/auth-ui/src/handlers"
	"gitlab.larvit.se/power-plan/auth-ui/src/utils"
)

func main() {
	log := utils.GetLog()

	err := godotenv.Load()
	if err != nil {
		log.Warn("Error loading .env file, this could be ok if the env file does not exist", "err", err.Error())
	}

	if os.Getenv("JWT_SHARED_SECRET") == "changeMe" {
		log.Error("JWT_SHARED_SECRET ENV is not set, using very insecure \"changeMe\"")
	}
	if os.Getenv("ADMIN_API_KEY") == "changeMe" {
		log.Error("ADMIN_API_KEY ENV is not set, using very insecure \"changeMe\"")
	}
	jwtSharedSecret := os.Getenv("JWT_SHARED_SECRET")
	adminApiKey := os.Getenv("ADMIN_API_KEY")
	apiUrl := os.Getenv("API_URL")

	app := fiber.New()

	apiInstance := api.Api{
		AdminApiKey:     adminApiKey,
		JwtSharedSecret: jwtSharedSecret,
		Log:             log,
		URL:             apiUrl,
	}

	handlers := h.Handlers{
		Api: apiInstance,
		Log: log,
	}

	// Log all requests
	app.Use(handlers.LogReq)

	// Static files
	app.Static("/", "./public")

	// Unpack user token, refresh token when needed etc.
	app.Use(handlers.HandleCookieTokens)

	app.Get("/", handlers.Index)
	app.Post("/", handlers.IndexPost)

	app.Get("/register", handlers.Register)
	app.Post("/register", handlers.RegisterPost)

	app.Post("/logout", handlers.LogoutPost)

	log.Info("Trying to start web server", "PORT", os.Getenv("PORT"))

	webBindHost := ":" + os.Getenv("PORT")
	err = app.Listen(webBindHost)
	for err != nil {
		log.Error("Could not start web server", "err", err.Error(), "webBindHost", webBindHost)
		time.Sleep(1 * time.Second)
		err = app.Listen(webBindHost)
	}

	log.Info("Web server closed, shutting down")
}
